import requests
from bs4 import BeautifulSoup


ESHOP_URL = "https://www.nintendo.com/games/detail/{slug}"


def scrap(slug):
    response = requests.get(
        url=ESHOP_URL.format(slug=slug),
        allow_redirects=True
    )

    if response.status_code != 200:
        return {}

    soup = BeautifulSoup(response.text, features="html.parser")
    div = soup.find("div", class_="game-details-container")
    script = [script for script in list(div.find_all("script")) if "window.game = Object.freeze" in str(script)][0]

    data = {}
    separator = ': "'

    for line in str(script).split("\n"):
        if separator not in line:
            continue

        key, value = line.strip().split(separator)
        value, _ = value.strip().split('"')

        data[key] = value

    languages = soup.find("dd", class_="languages")

    if languages:
        languages = languages.text.strip().split(", ")
    else:
        languages = ["English"]

    save_data_cloud = soup.find("styled-button", text="Save Data Cloud") is not None

    return {
        "nsuid": data.get("nsuid"),
        "product_code": data.get("productCode"),
        "slug": data["slug"].replace("\\u002D", "-"),
        "title": data["title"],
        "languages": languages,
        "save_data_cloud": save_data_cloud,
    }



from datetime import datetime
from typing import Dict

from nintendeals.commons.classes.games import Game
from nintendeals.commons.enumerates import Features, Platforms, Ratings, Regions

NSUIDS = {
    "500": Platforms.NINTENDO_3DS,
    "700": Platforms.NINTENDO_SWITCH,
    "200": Platforms.NINTENDO_WIIU,
}

PLATFORMS = {
    "CTR": Platforms.NINTENDO_3DS,
    "KTR": Platforms.NINTENDO_3DS,
    "HAC": Platforms.NINTENDO_SWITCH,
    "WUP": Platforms.NINTENDO_WIIU,
}


def build_game(data: Dict) -> Game:
    nsuid = data.get("nsuid_txt")
    product_code = data.get("product_code_txt")

    if nsuid:
        platform = NSUIDS[nsuid[:3]]
    else:
        platform = PLATFORMS[product_code[:3]]

    game = Game(
        platform=platform,
        region=Regions.EU,
        title=data["title"],
        nsuid=nsuid,
        product_code=product_code,
    )

    game.description = data.get("excerpt")
    game.slug = data.get("url")
    game.players = data.get("players_to", 0)
    game.free_to_play = data.get("price_regular_f") == 0.0

    # Release Date
    try:
        game.release_date = datetime.strptime(
            data.get("pretty_date_s"),
            "%d/%m/%Y"
        )
    except (ValueError, TypeError):
        game.release_date = None

    # Categories
    game.categories = data.get("game_categories_txt", [])

    # Developer
    developer = data.get("developer")
    game.developers = [developer] if developer else []

    # Languages
    languages = data.get("language_availability")
    game.languages = list(map(str.title, languages[0].split(","))) if languages else []

    # Publisher
    publisher = data.get("publisher")
    game.publishers = [publisher] if publisher else []

    # Rating (PEGI)
    game.rating = (Ratings.PEGI, data.get("age_rating_sorting_i"))

    # Features
    game.features = {
        Features.AMIIBO: data.get("near_field_comm_b", False),
        Features.DEMO: data.get("demo_availability", False),
        Features.DLC: data.get("add_on_content_b", False),
    }

    if game.platform == Platforms.NINTENDO_SWITCH:
        game.features[Features.NSO_REQUIRED] = data.get("paid_subscription_required_b", False)
        game.features[Features.SAVE_DATA_CLOUD] = data.get("cloud_saves_b", False)
        game.features[Features.VOICE_CHAT] = data.get("voice_chat_b", False)

    return game



from enum import Enum


class Features(str, Enum):
    AMIIBO = "Amiibo Supported"
    DEMO = "Demo Available"
    DLC = "DLC Available"
    NSO_REQUIRED = "Nintendo Switch Online Required"
    SAVE_DATA_CLOUD = "Save Data Cloud Supported"
    VOICE_CHAT = "Voice Chat Supported"

    def __str__(self):
        return str(self.value)


class Platforms(str, Enum):
    NINTENDO_WIIU = "Nintendo Wii U"
    NINTENDO_3DS = "Nintendo 3DS"
    NINTENDO_SWITCH = "Nintendo Switch"

    def __str__(self):
        return str(self.value)


class Ratings(str, Enum):
    CERO = "CERO"  # JP
    ESRB = "ESRB"  # NA
    PEGI = "PEGI"  # EU

    def __str__(self):
        return str(self.value)


class Regions(str, Enum):
    NA = "NA"
    EU = "EU"
    JP = "JP"

    def __str__(self):
        return str(self.value)



from datetime import datetime
from typing import Dict

from nintendeals.commons.classes.games import Game
from nintendeals.commons.enumerates import Features, Platforms, Ratings, Regions

PLATFORMS = {
    "4_WUP": Platforms.NINTENDO_WIIU,
    "2_CTR": Platforms.NINTENDO_3DS,
    "3_KTR": Platforms.NINTENDO_3DS,
    "1_HAC": Platforms.NINTENDO_SWITCH,
}

RATINGS = {
    "0": None,
    "1": "A",
    "2": "B",
    "3": "C",
    "4": "D",
    "5": "Z",
}


def build_game(data: Dict) -> Game:
    hard = data.get("hard")
    icode = data.get("icode")

    if hard and icode:
        product_code = hard[2:] + icode
    else:
        product_code = None

    game = Game(
        platform=PLATFORMS[hard],
        region=Regions.JP,
        title=data["title"],
        nsuid=data.get("nsuid"),
        product_code=product_code,
    )

    game.description = data.get("text")
    game.slug = data.get("icode")
    game.free_to_play = data.get("price") == 0.0

    # Players
    players = data.get("player") or ["0"]
    game.players = max(map(int, players[0].split("~")))

    # Release Date
    try:
        game.release_date = datetime.strptime(
            data.get("sdate"),
            "%Y.%m.%d"
        )
    except (ValueError, TypeError):
        game.release_date = None

    # Categories
    game.categories = data.get("genre", [])

    # Developer
    developer = data.get("maker")
    game.developers = [developer] if developer else []

    # Languages
    game.languages = data.get("lang", [])

    # Publisher
    publisher = data.get("publisher")
    game.publishers = [publisher] if publisher else []

    # Rating (CERO)
    rating = data.get("cero") or ["0"]
    game.rating = (Ratings.CERO, RATINGS.get(rating[0]))

    # Features
    game.features = {
        Features.AMIIBO: data.get("amiibo", "0") == "1",
    }

    if game.platform == Platforms.NINTENDO_SWITCH:
        game.features[Features.DLC] = len(data.get("cnsuid") or []) > 0
        game.features[Features.NSO_REQUIRED] = (data.get("nso") or ["0"]) == ["1"]

    return game



from .info import game_info

from .listing import (
    list_3ds_games,
    list_switch_games,
    list_wiiu_games,
)

from .search import (
    search_3ds_games,
    search_switch_games,
    search_wiiu_games,
)



from typing import Iterator, Optional

import requests

from nintendeals.commons.enumerates import Platforms

SEARCH_URL = "https://search.nintendo.jp/nintendo_soft/search.json"

HARDNAMES = {
    Platforms.NINTENDO_3DS: "2_CTR",
    Platforms.NINTENDO_SWITCH: "1_HAC",
    Platforms.NINTENDO_WIIU: "4_WUP",
}


def _search(query: str = "", platform: Platforms = None) -> Iterator[dict]:
    params = {
        "q": query,
        "limit": 150,
        "page": 0,
        "opt_hard": HARDNAMES.get(platform)
    }

    while True:
        params["page"] += 1
        response = requests.get(
            url=SEARCH_URL,
            params=params
        )

        if response.status_code != 200:
            break

        items = response.json()["result"]["items"]

        if not items:
            break

        yield from items


def search_by_nsuid(nsuid: str) -> Optional[dict]:
    try:
        return next(_search(query=nsuid))
    except StopIteration:
        return None


def search_by_platform(platform: Platforms) -> Iterator[dict]:
    yield from _search(platform=platform)


def search_by_query(query: str, platform: Platforms = None) -> Iterator[dict]:
    yield from _search(query=query, platform=platform)



from datetime import datetime
from typing import Any, Dict, List, Optional, Tuple, Union

from nintendeals.api.prices import get_price
from nintendeals.commons.classes.eshops import NAeShop, EUeShop, JPeShop
from nintendeals.commons.classes.prices import Price
from nintendeals.commons.enumerates import (
    Features,
    Platforms,
    Ratings,
    Regions,
)

ESHOPS = {
    Regions.NA: NAeShop,
    Regions.EU: EUeShop,
    Regions.JP: JPeShop,
}


class Game:

    def __init__(
        self,
        platform: Platforms,
        region: Regions,
        title: str,
        nsuid: str = None,
        product_code: str = None,
    ):
        self.platform: Platforms = platform
        self.region: Regions = region
        self.title: str = title
        self.nsuid: str = nsuid
        self.product_code: str = product_code

        self.description: Optional[str] = None
        self.slug: Optional[str] = None

        self.players: int = 0

        self.free_to_play: Optional[bool] = None

        self.release_date: Optional[datetime] = None

        self.categories: List[str] = []
        self.developers: List[str] = []
        self.languages: List[str] = []
        self.publishers: List[str] = []

        self.rating: Optional[Tuple[Ratings, Any]] = None

        self.features: Dict[Features, Any] = {}

    @property
    def unique_id(self) -> Optional[str]:
        if not self.product_code:
            return None

        if self.region == Regions.JP:
            return self.product_code[3:-1]
        else:
            return self.product_code[4:-1]

    @property
    def eshop(self) -> Union[NAeShop, EUeShop, JPeShop]:
        return ESHOPS[self.region](self)

    def price(self, country: str) -> Price:
        return get_price(country=country, game=self)

    def __repr__(self):
        return self.title



from typing import Iterator, Optional

import requests

from nintendeals.commons.enumerates import Platforms

SEARCH_URL = "https://search.nintendo-europe.com/en/select"

SYSTEM_NAMES = {
    Platforms.NINTENDO_3DS: "3DS",
    Platforms.NINTENDO_SWITCH: "Switch",
    Platforms.NINTENDO_WIIU: "Wii U",
}

PRODUCT_CODE_PREFIXES = "CTR", "KTR", "HAC", "WUP"
NSUIDS_PREFIXES = "200", "500", "700"


def _search(
        query: str = "*",
        nsuid: str = None,
        platform: Platforms = None
) -> Iterator[dict]:
    rows = 200

    params = {
        "fq": "type:GAME",
        "q": query,
        "rows": rows,
        "sort": "title asc",
        "start": -rows,
        "wt": "json",
    }

    if platform:
        system_name = SYSTEM_NAMES[platform]
        params["fq"] += f' AND system_names_txt:"{system_name}"'

    if nsuid:
        params["fq"] += f' AND nsuid_txt:"{nsuid}"'

    while True:
        params["start"] += rows
        response = requests.get(url=SEARCH_URL, params=params)

        if response.status_code != 200:
            break

        json = response.json()['response'].get('docs', [])

        if not len(json):
            break

        for data in json:
            nsuids = data.get("nsuid_txt", [])
            product_codes = data["product_code_txt"] = [
                pc.replace("-", "")
                    for pc in data.get("product_code_txt", [])
                        if pc[:3] in PRODUCT_CODE_PREFIXES
            ]

            if not any((nsuids, product_codes)):
                continue

            data["nsuid_txt"] = nsuids[0] if nsuids else 0
            data["product_code_txt"] = product_codes[0] if product_codes else 0

            yield data


def search_by_nsuid(nsuid: str) -> Optional[dict]:
    try:
        return next(_search(nsuid=nsuid))
    except StopIteration:
        return None


def search_by_platform(platform: Platforms) -> Iterator[dict]:
    yield from _search(platform=platform)


def search_by_query(query: str, platform: Platforms = None) -> Iterator[dict]:
    yield from _search(query=query, platform=platform)



from nintendeals.commons.enumerates import Platforms


class NAeShop:

    FORMAT = "https://www.nintendo.com/{lang}_{country}/games/detail/{slug}"

    def __init__(self, game: "Game"):
        self.game = game

    @property
    def ca_en(self) -> str:
        return NAeShop.FORMAT.format(
            lang="en",
            country="CA",
            slug=self.game.slug
        )

    @property
    def ca_fr(self) -> str:
        return NAeShop.FORMAT.format(
            lang="fr",
            country="CA",
            slug=self.game.slug
        )

    @property
    def us_en(self) -> str:
        return NAeShop.FORMAT.format(
            lang="en",
            country="US",
            slug=self.game.slug
        )


class EUeShop:

    FORMAT_NO_LANG = "https://www.nintendo.{domain}{slug}"
    FORMAT_LANG = "https://www.nintendo.{domain}/{lang}{slug}"

    FORMAT_ALT = "https://ec.nintendo.com/{country}/{lang}/titles/{nsuid}"

    def __init__(self, game: "Game"):
        self.game = game

    @property
    def at_de(self) -> str:
        return EUeShop.FORMAT_NO_LANG.format(
            domain="at",
            slug=self.game.slug
        )

    @property
    def be_fr(self) -> str:
        return EUeShop.FORMAT_LANG.format(
            domain="be",
            lang="fr",
            slug=self.game.slug
        )

    @property
    def be_nl(self) -> str:
        return EUeShop.FORMAT_LANG.format(
            domain="be",
            lang="nl",
            slug=self.game.slug
        )

    @property
    def ch_de(self) -> str:
        return EUeShop.FORMAT_LANG.format(
            domain="ch",
            lang="de",
            slug=self.game.slug
        )

    @property
    def ch_fr(self) -> str:
        return EUeShop.FORMAT_LANG.format(
            domain="ch",
            lang="fr",
            slug=self.game.slug
        )

    @property
    def ch_it(self) -> str:
        return EUeShop.FORMAT_LANG.format(
            domain="ch",
            lang="it",
            slug=self.game.slug
        )

    @property
    def de_de(self) -> str:
        return EUeShop.FORMAT_NO_LANG.format(
            domain="de",
            slug=self.game.slug
        )

    @property
    def es_es(self) -> str:
        return EUeShop.FORMAT_NO_LANG.format(
            domain="es",
            slug=self.game.slug
        )

    @property
    def fr_fr(self) -> str:
        return EUeShop.FORMAT_NO_LANG.format(
            domain="fr",
            slug=self.game.slug
        )

    @property
    def it_it(self) -> str:
        return EUeShop.FORMAT_NO_LANG.format(
            domain="it",
            slug=self.game.slug
        )

    @property
    def nl_nl(self) -> str:
        return EUeShop.FORMAT_NO_LANG.format(
            domain="nl",
            slug=self.game.slug
        )

    @property
    def pt_pt(self) -> str:
        return EUeShop.FORMAT_NO_LANG.format(
            domain="pt",
            slug=self.game.slug
        )

    @property
    def ru_ru(self) -> str:
        return EUeShop.FORMAT_LANG.format(
            domain="ru",
            lang="-",
            slug=self.game.slug
        )

    @property
    def uk_en(self) -> str:
        return EUeShop.FORMAT_NO_LANG.format(
            domain="co.uk",
            slug=self.game.slug
        )

    @property
    def za_en(self) -> str:
        return EUeShop.FORMAT_NO_LANG.format(
            domain="co.za",
            slug=self.game.slug
        )

    @property
    def au_en(self) -> str:
        if self.game.platform != Platforms.NINTENDO_SWITCH:
            raise ValueError("Only available for Nintendo Switch games")

        return EUeShop.FORMAT_ALT.format(
            country="AU",
            lang="en",
            nsuid=self.game.nsuid
        )

    @property
    def nz_en(self) -> str:
        if self.game.platform != Platforms.NINTENDO_SWITCH:
            raise ValueError("Only available for Nintendo Switch games")

        return EUeShop.FORMAT_ALT.format(
            country="NZ",
            lang="en",
            nsuid=self.game.nsuid
        )


class JPeShop:

    NEW_FORMAT = "https://store-jp.nintendo.com/list/software/{nsuid}.html"
    OLD_FORMAT = "https://www.nintendo.co.jp/titles/{nsuid}"

    def __init__(self, game: "Game"):
        self.game = game

    @property
    def jp_jp(self) -> str:
        if self.game.platform == Platforms.NINTENDO_3DS:
            return JPeShop.OLD_FORMAT.format(nsuid=self.game.nsuid)

        if self.game.platform == Platforms.NINTENDO_WIIU:
            return JPeShop.OLD_FORMAT.format(nsuid=self.game.nsuid)

        return JPeShop.NEW_FORMAT.format(nsuid=self.game.nsuid)



from typing import Optional

from nintendeals.commons.classes.games import Game
from nintendeals.noe.api import nintendo
from nintendeals.noe.util import build_game


def game_info(nsuid: str) -> Optional[Game]:
    """
    Given a game's `nsuid` for the EU region, it will retrieve its information
    from Nintendo of Europe.

    Available Features
    ------------------
        * Nintendo 3DS
            - AMIIBO
            - DEMO
            - DLC
        * Nintendo WiiU
            - AMIIBO
            - DEMO
            - DLC
        * Nintendo Switch
            - AMIIBO
            - DEMO
            - DLC
            - NSO_REQUIRED
            - SAVE_DATA_CLOUD
            - VOICE_CHAT

    Parameters
    ----------
    nsuid: str
        Valid nsuid of a nintendo game.

    Returns
    -------
    nintendeals.classes.common.Game:
        Information of the game.
    """
    data = nintendo.search_by_nsuid(nsuid)

    return build_game(data) if data else None



from datetime import datetime
from typing import Iterable, Iterator, Optional, Tuple

import requests
from dateutil.parser import parse as date_parser

from nintendeals.commons.classes.prices import Price


def _parse_date(string: str) -> datetime:
    return date_parser(string).replace(tzinfo=None)


def fetch_prices(
    country: str,
    nsuids: Iterable[str]
) -> Iterator[Tuple[str, Price]]:
    nsuids = set(nsuids)

    if not 51 > len(nsuids) > 0:
        raise ValueError("The amount of nsuids must between 1 and 50.")

    response = requests.get(
        url="https://api.ec.nintendo.com/v1/price",
        params=dict(
            country=country,
            lang="en",
            ids=",".join(nsuids),
        )
    )

    response.raise_for_status()

    for data in response.json().get('prices', []):
        nsuid = str(data["title_id"])
        regular_price = data.get("regular_price")

        if not regular_price:
            continue

        price = Price(
            nsuid=nsuid,
            country=country,
            currency=regular_price["currency"],
            value=float(regular_price["raw_value"]),
        )

        discount_price = data.get("discount_price")

        if discount_price:
            price.sale_value = float(discount_price["raw_value"])
            price.sale_start = _parse_date(discount_price['start_datetime'])
            price.sale_end = _parse_date(discount_price['end_datetime'])

        yield price.nsuid, price


def get_prices(games: Iterable["Game"], country: str) -> Iterator[Tuple[str, Price]]:
    """
    Given an iterable of `games` (each with a nsuid) and a valid `country` code
    it will retrieve the current pricing of those games for that country.

    Parameters
    ----------
    games: Iterable[nintendeals.commons.classes.games.Game]
        Games to get their pricing.
    country: str
        Valid alpha-2 code of the country.

    Yields
    -------
    Tuple[str, nintendeals.commons.classes.prices.Price]
        Dictionary containing the nsuids as keys and each pricing as value.
    """
    prices = {}
    chunk = []

    for game in games:
        chunk.append(game)

        if len(chunk) == 50:
            fetched = {
                nsuid: price for nsuid, price in
                fetch_prices(
                    country=country,
                    nsuids=[game.nsuid for game in chunk]
                )
            }
            prices.update(fetched)

            chunk = []

    if chunk:
        fetched = {
            nsuid: price for nsuid, price in
            fetch_prices(
                country=country,
                nsuids=[game.nsuid for game in chunk]
            )
        }
        prices.update(fetched)

    yield from prices.items()


def get_price(game: "Game", country: str) -> Optional[Price]:
    """
    Given a `game` (with an nsuid) and a valid `country` code it will retrieve
    the current pricing of that game for that country.

    Parameters
    ----------
    game: nintendeals.commons.classes.games.Game
        Game to get its pricing.
    country: str
        Valid alpha-2 code of the country.

    Returns
    -------
    nintendeals.classes.commons.prices.Price
        Pricing of the game in the indicated country.
    """
    fetched = {
        nsuid: price
        for nsuid, price in fetch_prices(country=country, nsuids=[game.nsuid])
    }

    return fetched.get(game.nsuid)



from datetime import datetime


class Price:

    def __init__(
        self,
        nsuid: str,
        country: str,
        currency: str,
        value: float
    ):
        self.nsuid: str = nsuid

        self.country: str = country
        self.currency: str = currency
        self.value: float = value

        self.sale_value: float = None
        self.sale_start: datetime = None
        self.sale_end: datetime = None

    @property
    def sale_discount(self) -> int:
        if not self.on_sale:
            return 0

        return round(100 * self.sale_value / self.value)

    @property
    def is_free_to_play(self) -> bool:
        return self.value == 0.0

    @property
    def is_sale_active(self) -> bool:
        if not self.on_sale:
            return False

        now = datetime.utcnow()
        return self.sale_end > now > self.sale_start

    @property
    def on_sale(self) -> bool:
        return self.sale_value is not None

    def __repr__(self) -> str:
        if self.on_sale:
            return f"{self.currency} {self.sale_value}*"

        return f"{self.currency} {self.value}"



from typing import Iterator

from nintendeals.commons.classes.games import Game
from nintendeals.commons.enumerates import Platforms
from nintendeals.noj.api import nintendo
from nintendeals.noj.util import build_game


def list_games(platform: Platforms) -> Iterator[Game]:
    for data in nintendo.search_by_platform(platform):
        yield build_game(data)


def list_3ds_games() -> Iterator[Game]:
    """
    Get a list of Nintendo 3DS games for the JP region.

    Available Features
    ------------------
        * AMIIBO
        * DLC
        * NSO_REQUIRED

    Yields
    -------
    nintendeals.classes.common.Game:
        Information of a game.
    """
    yield from list_games(Platforms.NINTENDO_3DS)


def list_switch_games() -> Iterator[Game]:
    """
    Get a list of Nintendo WiiU games for the JP region.

    Available Features
    ------------------
        * AMIIBO

    Yields
    -------
    nintendeals.classes.common.Game:
        Information of a game.
    """
    yield from list_games(Platforms.NINTENDO_SWITCH)


def list_wiiu_games() -> Iterator[Game]:
    """
    Get a list of Nintendo WiiU games for the JP region.

    Available Features
    ------------------
        * AMIIBO

    Yields
    -------
    nintendeals.classes.common.Game:
        Information of a game.
    """
    yield from list_games(Platforms.NINTENDO_WIIU)



from typing import Optional

from nintendeals.commons.classes.games import Game
from nintendeals.noa.api import algolia
from nintendeals.noa.scrapers import nintendo
from nintendeals.noa.util import build_game


def game_info_by_nsuid(nsuid: str) -> Optional[Game]:
    data = algolia.search_by_nsuid(nsuid)

    if not data:
        return None

    data["extra"] = nintendo.scrap(data["slug"])

    return build_game(data)


def game_info_by_slug(slug: str) -> Optional[Game]:
    extra = nintendo.scrap(slug)
    nsuid = extra.get("nsuid")

    if not nsuid:
        return None

    data = algolia.search_by_nsuid(nsuid)

    if not data:
        return None

    data["extra"] = extra

    return build_game(data)


def game_info(nsuid: str = None, slug: str = None) -> Optional[Game]:
    """
    Given a game's `nsuid` or url `slug` for the NA region, it will retrieve
    its information from Nintendo of America.

    Available Features
    ------------------
        * Nintendo 3DS
            - DEMO
        * Nintendo WiiU
            - DEMO.
        * Nintendo Switch
            - DEMO
            - DLC
            - NSO_REQUIRED
            - SAVE_DATA_CLOUD

    Parameters
    ----------
    nsuid: str
        Valid nsuid of a nintendo game.
    slug: str
        Valid slug from NA's eShop.

    Returns
    -------
    nintendeals.classes.common.Game:
        Information of the game.
    """
    if nsuid:
        return game_info_by_nsuid(nsuid)

    if slug:
        return game_info_by_slug(slug)

    return None



from typing import Iterator, Optional

from algoliasearch.search_client import SearchClient

from nintendeals.commons.enumerates import Platforms

APP_ID = "U3B6GR4UA3"
API_KEY = "c4da8be7fd29f0f5bfa42920b0a99dc7"

INDEX_NAME = "ncom_game_en_us"
INDEX = None


PLATFORMS = {
    Platforms.NINTENDO_3DS: "Nintendo 3DS",
    Platforms.NINTENDO_SWITCH: "Nintendo Switch",
    Platforms.NINTENDO_WIIU: "Wii U",
}

PLATFORM_CODES = {
    Platforms.NINTENDO_3DS: "5001",
    Platforms.NINTENDO_SWITCH: "7001",
    Platforms.NINTENDO_WIIU: "Wii U",
}


def _search_index(query, **options):
    global INDEX

    if not INDEX:
        client = SearchClient.create(APP_ID, API_KEY)
        INDEX = client.init_index(INDEX_NAME)

    response = INDEX.search(query, request_options=options)
    return response.get('hits', [])


def search_by_nsuid(nsuid: str) -> Optional[dict]:
    hits = _search_index(nsuid, restrictSearchableAttributes=['nsuid'])
    return (hits or [{}])[0]


def search_by_platform(platform: Platforms) -> Iterator[dict]:
    empty_pages = 0

    platform_code = PLATFORM_CODES[platform]

    options = {
        "allowTyposOnNumericTokens": False,
        "queryType": "prefixAll",
        "restrictSearchableAttributes": ["nsuid"],
        "facetFilters": [
            f"platform:{PLATFORMS[platform]}"
        ],
        "hitsPerPage": 500,
    }

    current = -1

    while True:
        current += 1
        query = f"{platform_code}{current:07}"
        games = _search_index(query, **options)

        if not games:
            empty_pages += 1

        if empty_pages == 5:
            break

        yield from games


def search_by_query(query: str, platform: Platforms = None) -> Iterator[dict]:
    hits_per_page = 50

    options = {
        "hitsPerPage": hits_per_page,
    }

    if platform:
        options["facetFilters"] = [
            f"platform:{PLATFORMS[platform]}"
        ]

    page = -1

    while True:
        page += 1
        options["page"] = page

        games = _search_index(query, **options)

        yield from games

        if len(games) < hits_per_page:
            break



import re
from datetime import datetime
from typing import Dict

from nintendeals.commons.classes.games import Game
from nintendeals.commons.enumerates import Features, Platforms, Ratings, Regions

PLATFORMS = {
    "Wii U": Platforms.NINTENDO_WIIU,
    "Nintendo 3DS": Platforms.NINTENDO_3DS,
    "Nintendo Switch": Platforms.NINTENDO_SWITCH,
}


def build_game(data: Dict) -> Game:
    extra = data.get("extra", {})

    game = Game(
        platform=PLATFORMS[data["platform"]],
        region=Regions.NA,
        title=data["title"],
        nsuid=data.get("nsuid"),
        product_code=extra.get("product_code"),
    )

    game.description = data.get("description")
    game.slug = data.get("slug")
    game.free_to_play = data.get("free_to_start", False)

    # Players
    try:
        game.players = int(re.sub(r"[^\d]*", "", data["numOfPlayers"]))
    except (KeyError, ValueError):
        game.players = 0

    # Release Date
    try:
        release_date = data["releaseDateDisplay"].split("T")[0]
        game.release_date = datetime.strptime(release_date, '%Y-%m-%d')
    except (KeyError, ValueError):
        game.release_date = None

    # Categories
    game.categories = data.get("genres", [])

    # Developer
    game.developers = data.get("developers", [])

    # Languages
    game.languages = extra.get("languages", [])

    # Publisher
    game.publishers = data.get("publishers", [])

    # Rating (ESRB)
    game.rating = (Ratings.ESRB, data.get("esrbRating"))

    # Features
    filters = data.get("generalFilters", [])

    game.features = {
        Features.DEMO: "Demo available" in filters,
    }

    if game.platform == Platforms.NINTENDO_SWITCH:
        game.features[Features.DLC] = "DLC available" in filters
        game.features[Features.NSO_REQUIRED] = "Nintendo Switch Online compatible" in filters
        game.features[Features.SAVE_DATA_CLOUD] = extra.get("save_data_cloud")

    return game



from os import path

from setuptools import setup

this_directory = path.abspath(path.dirname(__file__))

with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="nintendeals",
    version="2.1.0",
    url="https://github.com/fedecalendino/nintendeals",
    license="MIT",
    description="Scraping tools for Nintendo games and prices on NA, EU and JP.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Fede Calendino",
    author_email="fede@calendino.com",
    packages=[
        "nintendeals",
        "nintendeals.api",
        "nintendeals.commons",
        "nintendeals.commons.classes",
        "nintendeals.noa",
        "nintendeals.noa.api",
        "nintendeals.noa.scrapers",
        "nintendeals.noe",
        "nintendeals.noe.api",
        "nintendeals.noj",
        "nintendeals.noj.api",
    ],
    install_requires=[
        "algoliasearch",
        "beautifulsoup4",
        "pycountry",
        "python-dateutil",
        "requests",
        "xmltodict",
    ],
    keywords=[
        "nintendo",
        "eshop",
        "deals",
    ],
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)



from typing import Iterator

from nintendeals.commons.classes.games import Game
from nintendeals.commons.enumerates import Platforms
from nintendeals.noe.api import nintendo
from nintendeals.noe.util import build_game


def search_games(query: str, platform: Platforms) -> Iterator[Game]:
    for data in nintendo.search_by_query(query, platform):
        yield build_game(data)


def search_3ds_games(query: str) -> Iterator[Game]:
    """
    Search for Nintendo 3DS games in the EU region.

    Available Features
    ------------------
        * AMIIBO

    Parameters
    ----------
    query: str
        Text to search.

    Yields
    -------
    nintendeals.classes.common.Game:
        Information of a game.
    """
    yield from search_games(query, Platforms.NINTENDO_3DS)


def search_switch_games(query: str) -> Iterator[Game]:
    """
    Search for Nintendo Switch games in the EU region.

    Available Features
    ------------------
        * AMIIBO
        * DEMO
        * DLC
        * NSO_REQUIRED
        * SAVE_DATA_CLOUD
        * VOICE_CHAT

    Parameters
    ----------
    query: str
        Text to search.

    Yields
    -------
    nintendeals.classes.common.Game:
        Information of a game.
    """
    yield from search_games(query, Platforms.NINTENDO_SWITCH)


def search_wiiu_games(query: str) -> Iterator[Game]:
    """
    Search for Nintendo WiiU games in the EU region.

    Available Features
    ------------------
        * AMIIBO
        * DEMO
        * DLC

    Parameters
    ----------
    query: str
        Text to search.

    Yields
    -------
    nintendeals.classes.common.Game:
        Information of a game.
    """
    yield from search_games(query, Platforms.NINTENDO_WIIU)
